package com.simpledbbook.tools;
//TODO: options: -a <accountName> -s <serviceUrl>

import java.io.*;
import java.util.*;

import com.simpledbbook.*;

public abstract class AdminTool {
   static String USAGE = "java AdminTool [-l] [-m] [-c] [-d] [domain]";
   protected SimpleDB simpleDb;
   protected String domain;

   public static void main(String[] argz) {
      AdminTool tool = createInstance(argz);
      tool.invoke();
   }

   private static AdminTool createInstance(String[] argz) {
      try {
         String cmd = argz[0];
         if (cmd.equals("-l")) {
            return new ListingTool();
         } else if (cmd.equals("-m")) {
            return new MetadataTool(argz[1]);
         } else if (cmd.equals("-c")) {
            return new CreateTool(argz[1]);
         } else if (cmd.equals("-d")) {
            return new DeleteTool(argz[1]);
         }
      } catch (Exception e) {}
      System.err.println(USAGE);
      System.exit(-1);
      return null;
   }

   abstract void doInvoke();

   private void invoke() {
      try {
         loadCredentials();
         doInvoke();
      } catch (Exception e) {
         System.out.println(e.getMessage());
      }
   }

   private void loadCredentials() {
      try {
         initSimpledbFromDisk();
      } catch (Exception e) {
         String errorMsg = "credentials not found in $HOME/.awssecret";
         throw new RuntimeException(errorMsg);
      }
   }

   private void initSimpledbFromDisk() throws Exception {
      Properties props = new Properties();
      String home = System.getProperty("user.home");
      props.load(new FileReader(new File(home, ".awssecret")));
      String accessKeyId = props.getProperty("AWS_ACCESS_KEY_ID");
      String secretAccessKey = props.getProperty("AWS_SECRET_ACESSS_KEY");
      simpleDb = new SimpleDB(accessKeyId, secretAccessKey);
   }

   protected AdminTool(String domainName) {
      this.domain = domainName;
   }
}
