package com.simpledbbook.tools;

import static java.lang.System.*;

import java.util.*;

public class MetadataTool extends AdminTool {

   private static final double ONE_GB = 107374182.40D;

   public MetadataTool(String domainName) {
      super(domainName);
   }

   @Override
   void doInvoke() {
      Map<String, Integer> info = simpleDb.domainMetadata(domain);
      long epochMillis = info.get("Timestamp") * 1000L;
      long deltaMillis = System.currentTimeMillis() - epochMillis;
      double deltaHours = deltaMillis / 3600000D;
      int items = info.get("ItemCount");
      int itemNameBytes = info.get("ItemNamesSizeBytes");
      int attrNames = info.get("AttributeNameCount");
      int attrNameBytes = info.get("AttributeNamesSizeBytes");
      int attrValues = info.get("AttributeValueCount");
      int attrValueBytes = info.get("AttributeValuesSizeBytes");
      long totalBytes = itemNameBytes + attrNameBytes + attrValueBytes;
      double totalGB = totalBytes/ONE_GB;

      String headerA = "'%s' using %.2f%% of 10GB capacity";
      String headerB = " (as of %.1f hours ago)\n";
      out.printf(headerA + headerB, domain, totalGB, deltaHours);

      String lineFmt = "%16s: %,10d (%,d bytes)\n";
      out.printf(lineFmt, "Item names", items, itemNameBytes);
      out.printf(lineFmt, "Attribute names", attrNames, attrNameBytes);
      out.printf(lineFmt, "Attribute values", attrValues, attrValueBytes);
   }
}
