package com.simpledbbook.user;

import java.util.*;

import com.simpledbbook.*;

public class SimpleDBUserService {
   private static final String AUTH_NAMES = "authorities";
   private boolean enableGroups;
   private SimpleDB sdb;
   private String domainName = "users";

   public User loadUserByUsername(String username) {
      Item item = sdb.getItem(domainName, username);
      List<String> auth = getAuthorities(item);
      String pass = item.getAttribute("password");
      String enabledString = item.getAttribute("enabled");
      boolean enabled = enabledString.equals("true");
      return new User(username, pass, enabled, auth);
   }

   private List<String> getAuthorities(Item item) {
      if (!enableGroups) {
         return item.getAttributes(AUTH_NAMES);
      }
      List<String> groupNames = item.getAttributes("group");
      String query = buildGroupQuery(groupNames);
      return sdb.singleAttribueSelect(query, AUTH_NAMES);
   }

   private String buildGroupQuery(List<String> groupNames) {
      String f = "SELECT '%s' FROM `%s` WHERE itemName() IN(%s)";
      String groups = listToQueryString(groupNames);
      return String.format(f, AUTH_NAMES, domainName, groups);
   }

   private String listToQueryString(List<String> groupNames) {
      String result = "";
      for (String group : groupNames) {
         result = result + "," + group;
      }
      return result.substring(1);
   }

   public boolean isEnableGroups() {
      return enableGroups;
   }

   public void setEnableGroups(boolean groups) {
      enableGroups = groups;
   }

   public String getDomainName() {
      return domainName;
   }

   public void setDomainName(String domainName) {
      this.domainName = domainName;
   }
}
