using System;
using System.IO;
using System.Collections.Generic;
using Amazon.SimpleDB;
using Amazon.SimpleDB.Model;
using Attr = Amazon.SimpleDB.Model.Attribute;
using RequestAttr = Amazon.SimpleDB.Model.ReplaceableAttribute;

namespace ConsoleApplication1
{
    class Program
    {
private String id;
private String secret;

private void LoadCredentials()
{
    String home = (Environment.OSVersion.Platform == PlatformID.Unix || 
    Environment.OSVersion.Platform == PlatformID.MacOSX) ?
    Environment.GetEnvironmentVariable("HOME") :
    Environment.ExpandEnvironmentVariables("%HOMEDRIVE%%HOMEPATH%");
    Dictionary<String, String> data = new Dictionary<String, String>();
    String path = home + Path.DirectorySeparatorChar + ".awssecret";
    foreach (String row in File.ReadAllLines(path))
    {
        data.Add(row.Split('=')[0], row.Split('=')[1]);
    }
    id = data["AWS_ACCESS_KEY_ID"];
    secret = data["AWS_SECRET_ACESSS_KEY"];
}


        static void Main(string[] args)
        {
            Program app = new Program();
            app.InvokeAll();
        }

        private void InvokeAll()
        {
            LoadCredentials();
            //DeleteDomain();
            //CreateDomain();
            ListDomains();
            Metadata();
            PutAttributes();
            GetAttributes();
            DeleteAttributes();
            GetAttributes();
            BatchPut();
            Select();
            SelectWithNextToken();
            System.Threading.Thread.Sleep(50000);
        }

        private void CreateDomain()
        {
            AmazonSimpleDB simpleDB = new AmazonSimpleDBClient(id, secret);
            CreateDomainRequest request = new CreateDomainRequest();
            try
            {
                request.DomainName = "users";
                simpleDB.CreateDomain(request);

                request.DomainName = "products";
                simpleDB.CreateDomain(request);

                Console.WriteLine("create domain successful");
            }
            catch (AmazonSimpleDBException ex)
            {
                String code = ex.ErrorCode;
                String message = ex.Message;
                Console.WriteLine("create domain failed: {0}: {1}", code, message);
            }
        }

        private void ListDomains()
        {
AmazonSimpleDBConfig config = new AmazonSimpleDBConfig();
config.ServiceURL = "http://sdb.eu-west-1.amazonaws.com/";
AmazonSimpleDB simpleDB = new AmazonSimpleDBClient(id, secret, config);
ListDomainsRequest request = new ListDomainsRequest();
try
{
    ListDomainsResponse response = simpleDB.ListDomains(request);
    ListDomainsResult listDomainsResult = response.ListDomainsResult;
    foreach (String domainName in listDomainsResult.DomainName)
    {
        Console.WriteLine(domainName);
    }
}
catch (AmazonSimpleDBException ex)
{
    String code = ex.ErrorCode;
    String message = ex.Message;
    Console.WriteLine("list domain failed: {0}: {1}", code, message);
}
        }

        private void DeleteDomain()
        {
AmazonSimpleDB simpleDB = new AmazonSimpleDBClient(id, secret);
DeleteDomainRequest request = new DeleteDomainRequest();
try
{
    request.DomainName = "users";
    simpleDB.DeleteDomain(request);
    Console.WriteLine("delete domain successful");
}
catch (AmazonSimpleDBException ex)
{
    String code = ex.ErrorCode;
    String message = ex.Message;
    Console.WriteLine("delete domain failed: {0}: {1}", code, message);
}
        }

        private void Metadata()
        {
AmazonSimpleDB simpleDB = new AmazonSimpleDBClient(id, secret);
DomainMetadataRequest request = new DomainMetadataRequest();
String domain = "products";
try
{
    request.DomainName = domain;
    DomainMetadataResponse resp = simpleDB.DomainMetadata(request);
    DomainMetadataResult meta = resp.DomainMetadataResult;
    Console.WriteLine("'{0}': {1} items", domain, meta.ItemCount);
}
catch (AmazonSimpleDBException ex)
{
    String code = ex.ErrorCode;
    String message = ex.Message;
    Console.WriteLine("...failed: {0}: {1}", code, message);
}
        }

        private void PutAttributes()
        {
AmazonSimpleDB simpleDB = new AmazonSimpleDBClient(id, secret);
RequestAttr[] userData = new RequestAttr[]
{
    new RequestAttr().WithName("userid").WithValue("Mocky"),
    new RequestAttr().WithName("location").WithValue("Oswego"),
    new RequestAttr().WithName("tags").WithValue("SimpleDB"),
    new RequestAttr().WithName("tags").WithValue("Azure-Table"),
    new RequestAttr().WithName("tags").WithValue("App-Engine")
};
PutAttributesRequest request = new PutAttributesRequest();
try
{
    request.DomainName = "users";
    request.ItemName = "1234567";
    request.WithAttribute(userData);

    simpleDB.PutAttributes(request);
    Console.WriteLine("put attributes successful");
}
catch (AmazonSimpleDBException ex)
{
    String code = ex.ErrorCode;
    String message = ex.Message;
    Console.WriteLine("put attributes failed: {0}: {1}", code, message);
}
        }

        private void GetAttributes()
        {
AmazonSimpleDB simpleDB = new AmazonSimpleDBClient(id, secret);
GetAttributesRequest request = new GetAttributesRequest();
try
{
    request.DomainName = "users";
    request.ItemName = "1234567";
    GetAttributesResponse resp = simpleDB.GetAttributes(request);
    GetAttributesResult user = resp.GetAttributesResult;
    List<Attr> attrs = user.Attribute;

    String name = attrs.Find(
        delegate(Attr a) { return a.Name == "userid"; }).Value;
    String location = attrs.Find(
        delegate(Attr a) { return a.Name == "location"; }).Value;
    List<Attr> tags = attrs.FindAll(
        delegate(Attr a) { return a.Name == "tags"; });

    Console.Write("{0} ({1}) follows: ", name, location);
    tags.ForEach(
        delegate(Attr a) { Console.Write("{0} ", a.Value); });
    Console.WriteLine();
}
catch (AmazonSimpleDBException ex)
{
    String code = ex.ErrorCode;
    String message = ex.Message;
    Console.WriteLine("get attributes failed: {0}: {1}", code, message);
}
        }

        private void DeleteAttributes()
        {
AmazonSimpleDB simpleDB = new AmazonSimpleDBClient(id, secret);
DeleteAttributesRequest request = new DeleteAttributesRequest();
try
{
    request.DomainName = "users";
    request.ItemName = "1234567";
    request.WithAttribute(
        new Attr().WithName("tags").WithValue("App-Engine"));
    simpleDB.DeleteAttributes(request);
    Console.WriteLine("delete attributes successful");
}
catch (AmazonSimpleDBException ex)
{
    String code = ex.ErrorCode;
    String message = ex.Message;
    Console.WriteLine("delete attribues failed: {0}: {1}", code, message);
}
        }

        private void BatchPut()
        {
AmazonSimpleDB simpleDB = new AmazonSimpleDBClient(id, secret);
BatchPutAttributesRequest request = new BatchPutAttributesRequest();
try
{
    request.DomainName = "products";
    request.WithItem(buildRomeoCigar("Cigar001", "No. 2", "90"));
    request.WithItem(buildRomeoCigar("Cigar002", "Toro", "89"));
    request.WithItem(buildRomeoCigar("Cigar003", "Churchill", "94"));
    request.WithItem(buildRomeoCigar("Cigar004", "Robusto", "81"));

    simpleDB.BatchPutAttributes(request);
    Console.WriteLine("batch put successful");
}
catch (AmazonSimpleDBException ex)
{
    String code = ex.ErrorCode;
    String message = ex.Message;
    Console.WriteLine("batch put failed: {0}: {1}", code, message);
}
        }

private ReplaceableItem buildRomeoCigar(String id, String make, 
    String rating)
{
    RequestAttr[] attributes = new RequestAttr[] {
        new RequestAttr().WithName("ProductType").WithValue("Cigar"),
        new RequestAttr().WithName("Supplier").WithValue("Romeo y Julieta"),
        new RequestAttr().WithName("Make").WithValue(make),
        new RequestAttr().WithName("Rating").WithValue(rating),
    };
    ReplaceableItem item = new ReplaceableItem();
    item.ItemName = id;
    return item.WithAttribute(attributes);
}

        private void Select()
        {
AmazonSimpleDB simpleDB = new AmazonSimpleDBClient(id, secret);
SelectRequest request = new SelectRequest();
try
{
    request.SelectExpression = "SELECT * FROM users";
    printSelectResults(simpleDB.Select(request).SelectResult);
}
catch (AmazonSimpleDBException ex)
{
    String code = ex.ErrorCode;
    String message = ex.Message;
    Console.WriteLine("select failed: {0}: {1}", code, message);
}
        }

private void printSelectResults(SelectResult result) 
{
    foreach (Item item in result.Item) 
    {
        Console.WriteLine(item.Name);
        foreach (Attr attr in item.Attribute) 
        {
            Console.WriteLine("\t{0}", attr.Value);
        }
    }
}

        private void SelectWithNextToken()
        {
String select =
   "SELECT Rating, Make, Supplier" +
   "  FROM products" +
   " WHERE Rating > '85'" +
   "   AND ProductType = 'Cigar'" +
   " ORDER BY Rating DESC" +
   " LIMIT 2";

AmazonSimpleDB simpleDB = new AmazonSimpleDBClient(id, secret);
SelectRequest request = new SelectRequest();
try
{
    request.SelectExpression = select;
    SelectResult result = null;
    do
    {
        result = simpleDB.Select(request).SelectResult;
        printSelectResults(result);
        request.NextToken = result.NextToken;
    } while (result.IsSetNextToken());
}
catch (AmazonSimpleDBException ex)
{
    String code = ex.ErrorCode;
    String message = ex.Message;
    Console.WriteLine("select failed: {0}: {1}", code, message);
}
        }

    }
}
