package com.simpledbbook.core;

import java.io.*;
import java.net.*;

public class HTTPClient {
  private static final String CONTENT_TYPE = 
    "application/x-www-form-urlencoded; charset=utf-8";

  public Response fetch(Request request) {
    request.sign();
    URLConnection con = getConnection(request);
    writeRequest(request, con);
    String body = readInput(con);
    int code = getResponseCode(con);
    Response result = new Response(body, code);
    return result;
  }

  private URLConnection getConnection(Request request) {
    try {
      URLConnection con = request.getURL().openConnection();
      con.setRequestProperty("Content-Type", CONTENT_TYPE);
      String contentLength = request.getParamString().length() + "";
      con.setRequestProperty("Content-Length", contentLength);
      con.setDoOutput(true);
      con.connect();
      return con;
    } catch (IOException e) {
      throw new RuntimeException("Error opening connection", e);
    }
  }

  private void writeRequest(Request request, URLConnection con) {
    try {
      OutputStream outputStream = con.getOutputStream();
      OutputStreamWriter wr = new OutputStreamWriter(outputStream);
      wr.write(request.getParamString());
      wr.flush();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  private String readInput(URLConnection con) {
    InputStream in = null;
    try {
      in = con.getInputStream();
    } catch (Exception e) {
      HttpURLConnection http = (HttpURLConnection) con;
      in = http.getErrorStream();
    }
    return readFully(in);
  }

  private String readFully(InputStream in) {
    try {
      BufferedReader br = new BufferedReader(
          new InputStreamReader(in));
      StringBuilder page = new StringBuilder();
      String line = null;
      while ((line = br.readLine()) != null) {
        page.append(line);
      }
      in.close();
      return page.toString();
    } catch (IOException e) {
      throw new RuntimeException("Error reading from stream", e);
    }
  }

  private int getResponseCode(URLConnection con) {
    try {
      return ((HttpURLConnection)con).getResponseCode();
    } catch (IOException e) {
      return -1;
    }
  }
}
