package com.simpledbbook.tasklist;

import java.util.*;

import com.simpledbbook.SimpleDB;

public class TestTaskService {

  private static final String TEST_USER_DOMAIN = "users_01";
  private static final String TEST_TODO_DOMAIN = "tasks_01";
  private static final String TEST_USER_1 = "cZero169";
  private static final String TEST_USER_2 = "Tbuttersnaps";
  private static final String TEST_USER_3 = "mocky";
  private static final String TEST_PASSWORD = "fffff";

  public static void main(String[] args) throws InterruptedException {
    TestTaskService test = new TestTaskService();
    test.runTests();
  }

  private SimpleDB sdb;
  private TaskService app;
  private String user3sessionId;
  
  public TestTaskService() {
    sdb = new SimpleDB();
    app = new TaskService(sdb, TEST_USER_DOMAIN, TEST_TODO_DOMAIN);
    ensureDomainsExist();
  }

  private void ensureDomainsExist() {
    List<String> domains = sdb.listDomains();
    if (! domains.contains(TEST_USER_DOMAIN)) {
      sdb.createDomain(TEST_USER_DOMAIN);
    }
    if (! domains.contains(TEST_TODO_DOMAIN)) {
      sdb.createDomain(TEST_TODO_DOMAIN);
    }
  }

  private void runTests() {
    deleteUsers();
    pause();
    registerUsers();
    listUsers();
    addTasks();
    relog();
    deleteUsers();
  }

  private void relog() {
    pause();
    app.logOut(TEST_USER_3, user3sessionId);
    pause();
    try {
      List<String> tasksForToday = app.getTasksForDay(TEST_USER_3, TaskService.today(), user3sessionId);
      System.out.println(tasksForToday);
      assert false : "Access wrongly granted";
    } catch (AuthenticationFailure af) {
      // expected
    }
    user3sessionId = app.logIn(TEST_USER_3, TEST_PASSWORD).getSessionId();
    pause();
    List<String> tasksForToday = app.getTasksForDay(TEST_USER_3, TaskService.today(), user3sessionId);
    System.out.println(tasksForToday);
  }

  private void deleteUsers() {
    app.deleteUser(TEST_USER_1);
    app.deleteUser(TEST_USER_2);
    app.deleteUser(TEST_USER_3);
  }

  private void addTasks() {
    String today = TaskService.today();pause();
    addTask(today, "buy soap"); pause();
    addTask(today, "wash car"); pause();
    addTask(today, "wrk hard"); 
  }

  private void addTask(String today, String taskText) {
    long start = System.currentTimeMillis();
    app.addTask(TEST_USER_3, today, taskText, user3sessionId);
    long stop = System.currentTimeMillis();
    pause();
    List<String> tasksForToday = app.getTasksForDay(TEST_USER_3, today, user3sessionId);
    System.out.println(tasksForToday + "(" + (stop - start) + ")");
    ensureContains(tasksForToday, taskText);
  }

  private void pause() {
    try {
      Thread.sleep(3000);
    } catch (InterruptedException e) {
      e.printStackTrace();
    }
  }
  private void registerUsers() {
    app.registerNewUser(TEST_USER_1, TEST_PASSWORD);
    app.registerNewUser(TEST_USER_2, TEST_PASSWORD);
    user3sessionId = app.registerNewUser(TEST_USER_3, TEST_PASSWORD);
    pause();
  }

  private void listUsers() {
    List<String> users = app.listUsers();
    System.out.println(users);
    ensureContains(users, TEST_USER_1, TEST_USER_2, TEST_USER_3);
  }

  private void ensureContains(Collection<? extends Object> c, Object... values) {
    assert c.containsAll(Arrays.asList(values));
  }

}
